'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Watcher = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _dispersive = require('dispersive');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var isComponent = function isComponent(el) {
  return !!el && !!el.type && !!el.type.prototype && el.type.prototype instanceof _react.Component;
};

var Watcher = exports.Watcher = function (_Component) {
  _inherits(Watcher, _Component);

  function Watcher() {
    _classCallCheck(this, Watcher);

    return _possibleConstructorReturn(this, (Watcher.__proto__ || Object.getPrototypeOf(Watcher)).apply(this, arguments));
  }

  _createClass(Watcher, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      this.subscriptions = {};
      this.eachSource(function (_ref) {
        var name = _ref.name,
            source = _ref.source;
        return _this2.resetSubscription(name, source);
      });
    }
  }, {
    key: 'componentWillUpdate',
    value: function componentWillUpdate() {
      var _this3 = this;

      this.eachSource(function (_ref2) {
        var name = _ref2.name,
            source = _ref2.source;
        return _this3.resetSubscription(name, source);
      });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      var _this4 = this;

      this.eachSource(function (_ref3) {
        var name = _ref3.name;
        return _this4.removeSubscription(name);
      });
    }
  }, {
    key: 'getNewProps',
    value: function getNewProps() {
      var props = {};

      this.eachSource(function (_ref4) {
        var name = _ref4.name,
            source = _ref4.source;

        if (source instanceof _dispersive.Model) {
          props[name] = source.objects.get({ id: source.id });
        } else if (source instanceof _dispersive.EventEmitter.Emittable) {
          props[name] = source;
        } else {
          throw new Watcher.BadSourceType(name);
        }
      });

      return props;
    }
  }, {
    key: 'injectProps',
    value: function injectProps() {
      var _this5 = this;

      var filter = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var newProps = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var props = {};

      this.eachSource(function (_ref5) {
        var name = _ref5.name;

        if (!(name in filter) || filter[name] !== _this5.props.sources[name]) return;

        props[name] = newProps[name];
      });

      return props;
    }
  }, {
    key: 'removeSubscription',
    value: function removeSubscription(name) {
      if (!this.subscriptions[name]) return;

      this.subscriptions[name].remove();
      this.subscriptions[name] = null;
    }
  }, {
    key: 'resetSubscription',
    value: function resetSubscription(name, source) {
      var _this6 = this;

      this.removeSubscription(name);
      this.subscriptions[name] = source.changed(function () {
        return _this6.forceUpdate();
      });
    }
  }, {
    key: 'eachSource',
    value: function eachSource(predicate) {
      var _this7 = this;

      if (!this.props || !this.props.sources) return;

      Object.keys(this.props.sources).forEach(function (name) {
        return predicate({ name: name, source: _this7.props.sources[name] });
      });
    }
  }, {
    key: 'cloneElement',
    value: function cloneElement(el, newProps) {
      var _this8 = this;

      var children = el.props.children;

      if (_react2.default.Children.count(children) > 0 && !isComponent(el)) {
        children = _react2.default.Children.map(children, function (child) {
          return _this8.cloneElement(child, newProps);
        });
      }

      return _react2.default.cloneElement(el, this.injectProps(el.props, newProps), children);
    }
  }, {
    key: 'render',
    value: function render() {
      var childrenCount = _react2.default.Children.count(this.props.children);
      var newProps = this.getNewProps();

      if (childrenCount !== 1) throw new Watcher.BadChildrenCount(childrenCount);

      return this.cloneElement(_react2.default.Children.only(this.props.children), newProps);
    }
  }]);

  return Watcher;
}(_react.Component);

Watcher.propTypes = {
  sources: _react2.default.PropTypes.object.isRequired,
  children: _react2.default.PropTypes.element.isRequired
};

Watcher.BadChildrenCount = function WatcherBadChildrenCount(childrenCount) {
  Object.assign(this, {
    name: 'BadChildrenCount',
    message: 'Watcher accepts only one child. Received : ' + childrenCount
  });
};

Watcher.BadSourceType = function WatcherBadSourceType(name) {
  Object.assign(this, {
    name: 'BadSourceType',
    message: 'Watcher source not valid (bad type) : \'' + name + '\''
  });
};

